/*

*/

import tokenizer.*;
import tree.*;

/**
   This program evaluates (interprets) an
   Abstract Syntax Tree (AST) created by a parser.
*/
public class Evaluate
{
   /**
      This eval() method is essentially a post-order
      traversal of the abstract syntax tree.
   */
   public static int eval(final Tree<Token> tree)
   {
      final int result;

      if ( tree.isLeaf() )
      {
         result = Integer.parseInt( tree.root().lexeme );
      }
      else if ( 1 == tree.subTrees().size() )
      {
         final int  leftResult = eval( tree.subTrees().get(0) ); // recursion
         final String operator = tree.root().lexeme;
         if ( operator.equals("+") )
         {
            result = leftResult;
         }
         else if ( operator.equals("-") )
         {
            result = - leftResult;
         }
         else
         {
            throw new EvalError( "invalid unary operator: " + tree.root() );
         }
      }
      else if ( 2 == tree.subTrees().size() )
      {
         final int  leftResult = eval( tree.subTrees().get(0) ); // recursion
         final int rightResult = eval( tree.subTrees().get(1) ); // recursion
         final String operator = tree.root().lexeme;
         if ( operator.equals("+") )
         {
            result = leftResult + rightResult;
         }
         else if ( operator.equals("-") )
         {
            result = leftResult - rightResult;
         }
         else if ( operator.equals("*") )
         {
            result = leftResult * rightResult;
         }
         else if ( operator.equals("/") )
         {
            result = leftResult / rightResult;
         }
         else
         {
            throw new EvalError( "invalid binary operator: " + tree.root() );
         }
      }
      else
      {
         throw new EvalError( "invalid expression tree: " + tree );
      }

      return result;
   }
}
